@@ -9,7 +9,7 @@
 
 package Math::Formula;
 use vars '$VERSION';
-$VERSION = '0.10';
+$VERSION = '0.11';
 
 
 use warnings;
@@ -189,13 +189,15 @@ sub evaluate($)
 {	my ($self, $context, %args) = @_;
 	my $expr   = $self->expression;
 
-	my $result = ref $expr eq 'CODE'
-	  ? $self->toType($expr->($context, $self, %args))
-	  : $self->tree($expr)->_compute($context, $self);
+	my $result
+	  = ref $expr eq 'CODE' ? $self->toType($expr->($context, $self, %args))
+	  : ! blessed $expr     ? $self->tree($expr)->_compute($context, $self)
+	  : $expr->isa('Math::Formula::Type') ? $expr
+	  : panic;
 
 	# For external evaluation calls, we must follow the request
 	my $expect = $args{expect} || $self->returns;
-	$result && $expect && ! $result->isa($expect) ? $result->cast($expect) : $result;
+	$result && $expect && ! $result->isa($expect) ? $result->cast($expect, $context) : $result;
 }
 
 
