@@ -5,7 +5,7 @@
 
 package Math::Formula::Context;
 use vars '$VERSION';
-$VERSION = '0.10';
+$VERSION = '0.11';
 
 
 use warnings;
@@ -14,8 +14,6 @@ use strict;
 use Log::Report 'math-formula';
 use Scalar::Util qw/blessed/;
 
-my $config;
-
 
 sub new(%) { my $class = shift; (bless {}, $class)->init({@_}) }
 
@@ -32,12 +30,12 @@ sub init($)
 {	my ($self, $args) = @_;
 	my $name   = $self->{MFC_name}   = $args->{name} or error __x"context without a name";
 
-	my $config = $self->{MFC_config} = MF::FRAGMENT->new(config => $self, attributes => {
-		name       => MF::NAME->new($name),
+	$self->{MFC_attrs} = {
+		name       => MF::STRING->new($name),
 		mf_version => MF::STRING->new(undef, $Math::Formula::VERSION),
 		version    => $args->{version} ? MF::STRING->new($args->{version}) : undef,
 		created    => $self->_default(created => 'MF::DATETIME', $args->{created}, DateTime->now),
-	});
+	};
 
 	if(my $forms = $args->{formula})
 	{	$self->add(ref $forms eq 'ARRAY' ? @$forms : $forms);
@@ -50,10 +48,17 @@ sub init($)
 #--------------
 
 sub name   { $_[0]->{MFC_name} }
-sub config { $_[0]->{MFC_config} }
 
 #--------------
 
+sub attribute($)
+{	my ($self, $name) = @_;
+	my $def = $self->{MFC_attrs}{$name} or return;
+	Math::Formula->new($name => $def);
+}
+
+#XXX example with fragment
+
 sub add(@)
 {	my $self = shift;
 	unless(ref $_[0])
@@ -82,15 +87,17 @@ sub add(@)
 
 sub addFormula(@)
 {	my ($self, $first) = (shift, shift);
+	my $next = $_[0];
 
 	my ($name, $form) =
 	  !@_ && blessed $first && $first->isa('Math::Formula')
 	? ($first->name, $first)
-	: @_==1 && !ref $first && blessed $_[0] && $_[0]->isa('Math::Formula')
-	? ($first, $_[0])
-	: @_==1 && !ref $first && ref $_[0] eq 'ARRAY'
-	? ($first, Math::Formula->new($first, @{$_[0]}))
-	: @_ && !ref $first && !ref $_[0]
+	: @_==1 && !ref $first && blessed $next && $next->isa('Math::Formula')
+	? ($first, $next)
+	: @_==1 && !ref $first && ref $next eq 'ARRAY'
+	? ($first, Math::Formula->new($first, @{$next}))
+	: @_ && !ref $first &&
+		 (!ref $next || ref $next eq 'CODE' || blessed $next && $next->isa('Math::Formula::Type'))
 	? ($first, Math::Formula->new($first, @_))
 	: panic __x"formula declaration '{name}' not understood", name => $first;
 
@@ -114,11 +121,10 @@ sub fragment($) { $_[0]->{MFC_frags}{$_[1]} }
 sub evaluate($$%)
 {	my ($self, $name) = (shift, shift);
 
-	# Wow, I am impressed!
-	length $name or return $self->config;
+	# Wow, I am impressed!  Caused by prefix(#,.) -> infix
+	length $name or return $self;
 
-	# silently ignore missing tags
-	my $form = $self->formula($name);
+	my $form = $self->attribute($name) || $self->formula($name);
 	unless($form)
 	{	warning __x"no formula '{name}' in {context}", name => $name, context => $self->name;
 		return undef;
@@ -126,7 +132,8 @@ sub evaluate($$%)
 
 	my $claims = $self->{MFC_claims};
 	! $claims->{$name}++
-		or error __x"recursion in expression '{name}' at {context}", name => $name, context => $self->name;
+		or error __x"recursion in expression '{name}' at {context}",
+			name => $name, context => $self->name;
 
 	my $result = $form->evaluate($self, @_);
 
@@ -137,8 +144,14 @@ sub evaluate($$%)
 
 sub run($%)
 {	my ($self, $expr, %args) = @_;
-	my $name = delete $args{name} || join '#', (caller)[1,2];
-	Math::Formula->new($name, $expr)->evaluate($self, %args);
+	my $name  = delete $args{name} || join '#', (caller)[1,2];
+	my $result = Math::Formula->new($name, $expr)->evaluate($self, %args);
+
+	while($result && $result->isa('MF::NAME'))
+	{	$result = $self->evaluate($result->token, %args);
+	}
+
+	$result;
 }
 
 
