@@ -8,20 +8,19 @@ use v5.16;  # fc
 
 package Math::Formula::Type;
 use vars '$VERSION';
-$VERSION = '0.10';
+$VERSION = '0.11';
 
 use base 'Math::Formula::Token';
 
 #!!! The declarations of all other packages in this file are indented to avoid
 #!!! indexing by CPAN.
 
-use Log::Report 'math-formula', import => [ qw/error __x/ ];
+use Log::Report 'math-formula', import => [ qw/warning error __x/ ];
 
 # Object is an ARRAY. The first element is the token, as read from the formula
 # or constructed from a computed value.  The second is a value, which can be
 # used in computation.  More elements are type specific.
 
-# new() is implemented in the base-class, but documented here
 
 #-----------------
 
@@ -55,7 +54,11 @@ sub prefix
 		op => $op, child => $child;
 }
 
-sub attribute { warn "Cannot find attribute $_[1] for ".(ref $_[0]) }
+sub attribute {
+	warning __x"cannot find attribute '{attr}' for {class} '{token}'",
+		attr => $_[1], class => ref $_[0], token => $_[0]->token;
+	undef;
+}
 
 sub infix($@)
 {   my $self = shift;
@@ -63,7 +66,7 @@ sub infix($@)
 
 	if($op eq '.' && $right->isa('MF::NAME'))
 	{	if(my $attr = $self->attribute($right->token))
-		{	return $attr->($self, @_);
+		{	return ref $attr eq 'CODE' ? $attr->($self, @_) : $attr;
 		}
 	}
 
@@ -71,16 +74,14 @@ sub infix($@)
 	return $self->cast('MF::STRING', $context)->infix(@_)
 		if $op eq '~';
 
-	error __x"cannot find infix operator '{op}' for ({left} -> {right})",
+	error __x"cannot match infix operator '{op}' for ({left} -> {right})",
 		op => $op, left => ref $self, right => ref $right;
 }
 
 #-----------------
 
-	package MF::BOOLEAN;
-use vars '$VERSION';
-$VERSION = '0.10';
-
+package
+	MF::BOOLEAN;
 
 use base 'Math::Formula::Type';
 
@@ -113,16 +114,21 @@ sub _value($) { $_[1] eq 'true' }
 
 #-----------------
 
-	package MF::STRING;
-use vars '$VERSION';
-$VERSION = '0.10';
-
+package
+	MF::STRING;
 
 use base 'Math::Formula::Type';
 
 use Unicode::Collate ();
 my $collate = Unicode::Collate->new;  #XXX which options do we need?
 
+# Perl's false is 'undef': convert it to '0'
+sub new($$@)
+{	my ($class, $token, $value) = (shift, shift, shift);
+	$value //=0 unless defined $token;
+	$class->SUPER::new($token, $value, @_);
+}
+
 sub _token($) { '"' . ($_[1] =~ s/[\"]/\\$1/gr) . '"' }
 
 sub _value($)
@@ -136,8 +142,8 @@ sub _value($)
 sub cast($)
 {	my ($self, $to) = @_;
 
-	  $to eq 'MF::REGEXP'  ? MF::REGEXP->_from_string($_[0])
-	: $to eq 'MF::PATTERN' ? MF::PATTERN->_from_string($_[0])
+	  ref $self eq __PACKAGE__ && $to eq 'MF::REGEXP'  ? MF::REGEXP->_from_string($self)
+	: ref $self eq __PACKAGE__ && $to eq 'MF::PATTERN' ? MF::PATTERN->_from_string($self)
 	: $self->SUPER::cast($to);
 }
 
@@ -153,13 +159,14 @@ sub infix($$$)
 	{	my $r = $right->isa('MF::REGEXP') ? $right : $right->cast('MF::REGEXP', $context);
 		my $v = ! $r ? undef
 			  : $op eq '=~' ? $self->value =~ $r->regexp : $self->value !~ $r->regexp;
-		return MF::BOOLEAN->new(undef, $v // 0) if $r;
+		return MF::BOOLEAN->new(undef, $v) if $r;
 	}
 	elsif($op eq 'like' || $op eq 'unlike')
-	{	my $r = $right->isa('MF::PATTERN') ? $right : $right->cast('MF::PATTERN', $context);
+	{	# When expr is CODE, it may produce a qr// instead of a pattern.
+		my $r = $right->isa('MF::PATTERN') || $right->isa('MF::REGEXP') ? $right : $right->cast('MF::PATTERN', $context);
 		my $v = ! $r ? undef
 			  : $op eq 'like' ? $self->value =~ $r->regexp : $self->value !~ $r->regexp;
-		return MF::BOOLEAN->new(undef, $v // 0) if $r;
+		return MF::BOOLEAN->new(undef, $v) if $r;
 	}
 	elsif($op eq 'cmp')
 	{	my $r = $right->isa('MF::STRING') ? $right : $right->cast('MF::STRING', $context);
@@ -178,10 +185,8 @@ sub attribute($) { $string_attrs{$_[1]} || $_[0]->SUPER::attribute($_[1]) }
 
 #-----------------
 
-	package MF::INTEGER;
-use vars '$VERSION';
-$VERSION = '0.10';
-
+package
+	MF::INTEGER;
 
 use base 'Math::Formula::Type';
 use Log::Report 'math-formula', import => [ qw/error __x/ ];
@@ -253,10 +258,8 @@ sub attribute($) { $int_attrs{$_[1]} || $_[0]->SUPER::attribute($_[1]) }
 
 #-----------------
 
-	package MF::FLOAT;
-use vars '$VERSION';
-$VERSION = '0.10';
-
+package
+	MF::FLOAT;
 
 use base 'Math::Formula::Type';
 use POSIX  qw/floor/;
@@ -308,10 +311,8 @@ sub infix($$$)
 
 #-----------------
 
-	package MF::DATETIME;
-use vars '$VERSION';
-$VERSION = '0.10';
-
+package
+	MF::DATETIME;
 
 use base 'Math::Formula::Type';
 use DateTime ();
@@ -400,10 +401,8 @@ sub attribute($)
 
 #-----------------
 
-	package MF::DATE;
-use vars '$VERSION';
-$VERSION = '0.10';
-
+package
+	MF::DATE;
 
 use base 'Math::Formula::Type';
 
@@ -503,10 +502,8 @@ sub attribute($) { $date_attrs{$_[1]} || $_[0]->SUPER::attribute($_[1]) }
 
 #-----------------
 
-	package MF::TIME;
-use vars '$VERSION';
-$VERSION = '0.10';
-
+package
+	MF::TIME;
 use base 'Math::Formula::Type';
 
 use constant GIGA => 1_000_000_000; 
@@ -582,10 +579,8 @@ sub infix($$@)
 
 #-----------------
 
-	package MF::DURATION;
-use vars '$VERSION';
-$VERSION = '0.10';
-
+package
+	MF::DURATION;
 use base 'Math::Formula::Type';
 
 use DateTime::Duration ();
@@ -641,10 +636,8 @@ sub attribute($) { $dur_attrs{$_[1]} || $_[0]->SUPER::attribute($_[1]) }
 
 #-----------------
 
-	package MF::NAME;
-use vars '$VERSION';
-$VERSION = '0.10';
-
+package
+	MF::NAME;
 use base 'Math::Formula::Type';
 
 use Log::Report 'math-formula', import => [ qw/error __x/ ];
@@ -652,6 +645,8 @@ use Log::Report 'math-formula', import => [ qw/error __x/ ];
 my $pattern = '[_\p{Alpha}][_\p{AlNum}]*';
 sub _match() { $pattern }
 
+sub value($) { error __x"name '{name}' cannot be used as value.", name => $_[0]->token }
+
 
 sub validated($$)
 {	my ($class, $name, $where) = @_;
@@ -667,8 +662,8 @@ sub cast(@)
 {	my ($self, $type, $context) = @_;
 
 	if($type->isa('MF::FRAGMENT'))
-	{	my $frag = $context->fragment($self->token);
-		return $frag if $frag;
+	{	my $frag = $self->token eq '' ? $context : $context->fragment($self->token);
+		return MF::FRAGMENT->new($frag->name, $frag) if $frag;
 	}
 
 	$context->evaluate($self->token, expect => $type);
@@ -677,21 +672,37 @@ sub cast(@)
 sub infix(@)
 {	my $self = shift;
 	my ($op, $right, $context) = @_;
+	my $name = $self->token;
 
-	my $left = $context->evaluate($self->token);
-	$left ? $left->infix(@_) : undef;
+	if($op eq '.')
+	{	my $left = $name eq '' ? MF::FRAGMENT->new($context->name, $context) : $context->evaluate($name);
+		return $left->infix(@_) if $left;
+	}
+	if($op eq '#')
+	{	my $left = $name eq '' ? MF::FRAGMENT->new($context->name, $context) : $context->fragment($name);
+		return $left->infix(@_) if $left;
+	}
+
+	my $left = $context->evaluate($name);
+	$left ? $left->infix($op, $right, $context): undef;
 }
 
 #-----------------
 
-	package MF::PATTERN;
-use vars '$VERSION';
-$VERSION = '0.10';
- 
+package
+	MF::PATTERN; 
 use base 'MF::STRING';
 
+use Log::Report 'math-formula', import => [ qw/warning __x/ ];
+
+sub _token($) {
+	warning __x"cannot convert qr back to pattern, do {regexp}", regexp => $_[1];
+    "pattern meaning $_[1]";
+}
+
 sub _from_string($)
 {	my ($class, $string) = @_;
+	$string->token;        # be sure the pattern is kept as token: cannot be recovered
 	bless $string, $class;
 }
 
@@ -721,10 +732,8 @@ sub regexp() { $_[0][2] //= _to_regexp($_[0]->value) }
 
 #-----------------
 
-	package MF::REGEXP;
-use vars '$VERSION';
-$VERSION = '0.10';
-
+package
+	MF::REGEXP;
 use base 'MF::STRING';
 
 sub _from_string($)
@@ -742,49 +751,34 @@ sub regexp
 
 #-----------------
 
-	package MF::FRAGMENT;
-use vars '$VERSION';
-$VERSION = '0.10';
-
+package
+	MF::FRAGMENT;
 use base 'Math::Formula::Type';
 
 use Log::Report 'math-formula', import => [ qw/panic error __x/ ];
 
-
-sub new(@)
-{	my ($class, $name, $context, %args) = @_;
-
-	my $self = $class->SUPER::new($name, $context,
-		$args{attributes} || {},
-	);
-
-	$self;
-}
-
-sub name    { $_[0]->token }
-sub context { $_[0]->value }
+sub name    { $_[0][0] }
+sub context { $_[0][1] }
 
 sub infix($$@)
 {	my $self = shift;
 	my ($op, $right, $context) = @_;
+	my $name = $right->token;
 
-	if($op eq '#')
-	{	my $r = $right->isa('MF::FRAGMENT') ? $right : $right->cast('MF::FRAGMENT', $context);
-		return $r if $r;
+	if($op eq '#' && $right->isa('MF::NAME'))
+	{	my $fragment = $self->context->fragment($name)
+			or error __x"cannot find fragment '{name}' in '{context}'",
+				name => $name, context => $context->name;
+
+		return $fragment;
+	}
+
+	if($op eq '.' && $right->isa('MF::NAME'))
+	{	my $result = $self->context->evaluate($name);
+		return $result if $result;
 	}
 
 	$self->SUPER::infix(@_);
 }
 
-
-sub addAttribute($$)
-{	my ($self, $name, $code) = @_;
-	$self->[2]{$name} = $code;
-}
-
-sub attribute($)
-{	my ($self, $name) = @_;
-	$self->[2]{$name} || $self->SUPER::attribute($name);
-}
-
 1;
